
    def debug_mod12 
      ''
    end

    def cache_root_dir
      '/home/melezhik/projects/sparrow/.tom/.cache/tmp/967' 
    end

    def test_root_dir
      '/home/melezhik/projects/sparrow/.tom/.cache/tmp/967' 
    end

    def project_root_dir
      '/home/melezhik/projects/sparrow/.tom/.cache/plugins/public/bash' 
    end

    def cache_dir
      '/home/melezhik/projects/sparrow/.tom/.cache/tmp/967/story-0'
    end

    def story_dir
      '/home/melezhik/projects/sparrow/.tom/.cache/plugins/public/bash/'
    end

    def stdout_file
      '/home/melezhik/projects/sparrow/.tom/.cache/tmp/967/story-0/std.out' 
    end

