
package glue;
1;

package main;
use strict;
  
    sub debug_mod12 {
      
    }

    sub cach_root_dir {
      '/home/melezhik/projects/sparrow/.tom/.cache/tmp/5035'
    }

    sub test_root_dir {
      '/home/melezhik/projects/sparrow/.tom/.cache/tmp/5035'
    }

    sub project_root_dir {
      '/home/melezhik/projects/sparrow/.tom/.cache/plugins/public/bash' 
    }

    sub  cache_dir {
      '/home/melezhik/projects/sparrow/.tom/.cache/tmp/5035/story-1'
    }

    sub story_dir {
      '/home/melezhik/projects/sparrow/.tom/.cache/plugins/public/bash//modules/bash-command'
    }

    sub os { 'centos7' }


1;

