
    function debug_mod12 {
      ''
    }

    function cache_root_dir {
      '/home/melezhik/projects/sparrow/.tom/.cache/tmp/967'
    }

    function test_root_dir {
      '/home/melezhik/projects/sparrow/.tom/.cache/tmp/967'
    }

    function project_root_dir {
      '/home/melezhik/projects/sparrow/.tom/.cache/plugins/public/bash'
    }

    function cache_dir {
      '/home/melezhik/projects/sparrow/.tom/.cache/tmp/967/story-1'
    }

    function story_dir {
      '/home/melezhik/projects/sparrow/.tom/.cache/plugins/public/bash//modules/bash-command'
    }

    function stdout_file {
      '/home/melezhik/projects/sparrow/.tom/.cache/tmp/967/story-1/std.out'
    }

