
def debug_mod12():
  return 

def cache_root_dir():
  return '/home/melezhik/projects/sparrow/.tom/.cache/tmp/967' 

def test_root_dir():
  return '/home/melezhik/projects/sparrow/.tom/.cache/tmp/967' 

def project_root_dir():
  return '/home/melezhik/projects/sparrow/.tom/.cache/plugins/public/bash' 

def cache_dir():
  return '/home/melezhik/projects/sparrow/.tom/.cache/tmp/967/story-1'

def story_dir():
  return '/home/melezhik/projects/sparrow/.tom/.cache/plugins/public/bash//modules/bash-command'

def stdout_file():
  return '/home/melezhik/projects/sparrow/.tom/.cache/tmp/967/story-1/std.out' 

