
    debug_mod=debug_mod12 

    test_root_dir=/home/melezhik/projects/sparrow/.tom/.cache/tmp/967

    cache_root_dir=/home/melezhik/projects/sparrow/.tom/.cache/tmp/967

    project_root_dir=/home/melezhik/projects/sparrow/.tom/.cache/plugins/public/bash

    cache_dir=/home/melezhik/projects/sparrow/.tom/.cache/tmp/967/story-1

    story_dir=/home/melezhik/projects/sparrow/.tom/.cache/plugins/public/bash//modules/bash-command

    stdout_file=/home/melezhik/projects/sparrow/.tom/.cache/tmp/967/story-1/std.out 

    os=centos7

